

/* 

Temperature Sensor DS18B20 an Digitalen Port Pin 10 wie folgt verbunden
Links=Masse,  (Chip liegt auf Rundung; Schrift ist lesbar; Beine zeigen zu mir) 
Mitte=Data, 
Rechts=+5V, 
3300 to 4700 Ohm Widerstand von +5V nach Data.

Es wird erst die Adresse des 1-Wire-Device ausgegeben und dann wird die Temperaturmessung gestartet.



/* Benötigte Bibliotheken importieren */
#include <OneWire.h>
#include <DallasTemperature.h>
#include "max7219.h"
//Falls Bibliotheken fehlen: aus Internet Herunterladen und mit diesem Menüpunkt installieren: Sketch/Bibliothek importieren/Bibliothek hinzufügen...


#define ONE_WIRE_BUS 10 /* Digitalport Pin 10 definieren */

OneWire ourWire(ONE_WIRE_BUS); /* Ini oneWire instance */

DallasTemperature sensors(&ourWire);/* Dallas Temperature Library für Nutzung der oneWire Library vorbereiten */
// initialize the library with the numbers of the interface pins

/*
 Led controller MAX7219:
 pin 7 is connected to the DataIn 
 pin 6 is connected to the CLK 
 pin 5 is connected to the CS 
 */
MAX7219 led=MAX7219(7,6,5);
int history[720];
unsigned int index;
unsigned long time;
int minimum_temperature;

//-----------------------------------------------------------------------------
void writeDualValTo7Segment( int n1,  int n2, bool both) {
//-----------------------------------------------------------------------------
  //Suppresses leading zeros. Fills all 8 digits.
char neg1=' ';
char neg2=' ';
int n;
  if (n1<0) neg1 = '-';
  if (n2<0) neg2 = '-';
  n1 = abs(n1);
  n2 = abs(n2);

  if (!both) led.clearDisplay();
  n=n1;
  led.setChar(0, n%10, false);  n/=10;
  led.setChar(1, n%10, true);  n/=10;
  if (0==n){
    led.setChar(2, neg1, false);  
    neg1 = ' ';
  }else{ 
    led.setChar(2, n%10, false);  n/=10;
  }  
  led.setChar(3, neg1, false);  

  if (!both) return;
  
  n=n2;
  led.setChar(4, n%10, false);  n/=10;
  led.setChar(5, n%10, true);  n/=10;
  if (0==n){
    led.setChar(6, neg2, false);  
    neg2 = ' ';
  }else{ 
    led.setChar(6, n%10, false);  n/=10;
  }  
  led.setChar(7, neg2, false);  
}
//-----------------------------------------------------------------------------
void setup()
//-----------------------------------------------------------------------------
{
 
   /*
   The led controller MAX7219 is in power-saving mode on startup,
   we have to do a wakeup call
   */  
  led.shutdown(false);
  /* Set the brightness to a low value */
  led.setIntensity(4);

  sensors.begin();/* Initialisieren der Dallas Temperature library */
  sensors.setResolution(TEMP_12_BIT); // Genauigkeit auf 12-Bit setzen
  //Sensor 1:  0x28, 0xAA, 0x26, 0x97, 0x06, 0x00, 0x00, 0x5F
  for (int i=0; i<720; i++){
    history[i]=400;
  }
  time = 0L;
}

//-----------------------------------------------------------------------------
void loop()
//-----------------------------------------------------------------------------
{
  static byte tic=0;
  int temperature;
  unsigned long now;
  led.shutdown(false);
  sensors.requestTemperatures(); // Temp abfragen
  temperature = sensors.getTempCByIndex(0) * 10;
  
  
  writeDualValTo7Segment(temperature, 
                         minimum_temperature, (tic++ < 3));
  tic++;
  if (tic >8) tic=0;    
  delay(600);
  
  if (abs(millis() - time) >= 120000L){
    time += 120000L;
     history[index++] = temperature;
     if (index >719) index = 0;
     minimum_temperature = history[0];
     for (int i=0; i < 720; i++){
       if (history[i] < minimum_temperature) minimum_temperature = history[i];
     }
  }
 
}


