/* 


Temperatur Sensor DS18B20 in T=92 housing at digital Port Pin 10:
(DS18B20 lays on round side; writing on flat side is readable; pins directing to me) 
Left    = Gnd
Middle  = Data (Pin 10); connected via 3300...4700 Ohm resistor to +5V (Pull up). 
Right   = +5V 

ESP-8266: Arduino Tx via 1k2 to ESP-RX and further on via 2k2 to GND (converting 5V to 3.3V)
WLAN-Request:  api.openweathermap.org/data/2.5/forecast?id=2885163&units=metric&mode=xml&APPID=a6xxxxxxxxxxxxxxxxxxxxxxxxxxxx7c
*/

/* import libraries */
#include <OneWire.h>
#include <DallasTemperature.h>
#include "max7219.h"

#define SSID "my Router"         // insert your SSID
#define PASS "1234567890Abc"     // insert your password
#define LOCATIONID "2885163"     // location id
#define DST_IP "188.226.175.223" //api.openweathermap.org
#define APPID "a6xxxxxxxxxxxxxxxxxxxxxxxxxxxx7c"

#define ONE_WIRE_BUS 10 /* define digital port Pin 10 */
OneWire ourWire(ONE_WIRE_BUS); /* Init oneWire instance */
int forecast=0;
int err=0;

DallasTemperature sensors(&ourWire);/* prepare Dallas Temperature Library for oneWire processing */

/*
 Led controller MAX7219:
 pin 7 is connected to the DataIn 
 pin 4 is connected to the CLK 
 pin 5 is connected to the CS 
 */
// initialize the library with the numbers of the interface pins
MAX7219 led=MAX7219(7,4,5);

//-----------------------------------------------------------------------------
boolean SerialFind(char *str, unsigned long waitFirst, unsigned long waitNext)
//-----------------------------------------------------------------------------
//Waits up to "waitFirst" time for the first char and
//then "waitNext" time for the next char
//returns false on timeout, true on success
{
    unsigned long mytime;
    char *strtemp;
    char incoming;
    strtemp = str; 
    mytime = millis() + waitFirst;
    while ((mytime > millis()) || Serial.available()){ // if timer demands or there is something there
        if (Serial.available()){
            mytime = millis() + waitNext; 
            incoming = Serial.read(); 
            if (incoming==*strtemp) { 
                strtemp++; 
                if (*strtemp==0) {
                    return true;
                } 
            } else strtemp = str; 
        } 
   }
   return false;
}

//-----------------------------------------------------------------------------
void clearSerial(unsigned long duration_ms){
//-----------------------------------------------------------------------------
//reads char from serial for at least duration_ms time until no chars available.
unsigned long endTime;
   endTime = millis() + duration_ms;
    while ((endTime > millis()) || Serial.available()){ // if timeout or there are data to read
        if (Serial.available()) Serial.read();
    } 
}

//-----------------------------------------------------------------------------
void writeIntTo7Segment( int n, boolean leftSide) {
//-----------------------------------------------------------------------------
//Suppresses leading zeros. Fills all 4 digits.
char neg1=' ';
char neg2=' ';
char d0;
int offset=0;
if (leftSide) offset=4;
    if (n < 0) neg1 = '-';
    n = abs(n);

    d0=n%10;
    if (!leftSide) led.setChar(offset, d0, true); //blinking dot (indicating: Arduino alive)
    delay(200);
    led.setChar(offset, d0, false);
    offset++;

    n/=10;
    led.setChar(offset, n%10, true);  n/=10;
    offset++;
    if (0==n){
        led.setChar(offset, neg1, false);  
        neg1 = ' ';
    }else{ 
        led.setChar(offset, n%10, false);  n/=10;
    }  
    offset++;
    led.setChar(offset, neg1, false);  
}

//-----------------------------------------------------------------------------
boolean connectWiFi()
//-----------------------------------------------------------------------------
{
    Serial.println("AT+CWMODE=1");
    Serial.flush();
    String cmd="AT+CWJAP=\"";
    cmd+=SSID;
    cmd+="\",\"";
    cmd+=PASS;
    cmd+="\"";
    //dbgSerial.println(cmd);
    Serial.println(cmd);
    Serial.flush();
    delay(1000);
    if(Serial.find("OK")){
        //dbgSerial.println("OK, Connected to WiFi.");
        return true;
    }else{
        //dbgSerial.println("Can not connect to the WiFi.");
        delay(2000);
        return false;
    }
}

//-----------------------------------------------------------------------------
boolean getForecast()  //returns true on success. REsult is stored in  global var "forecast"
//-----------------------------------------------------------------------------
{
    static String inString = "";    // string to hold input
    static unsigned long t1;

    Serial.println("AT+RST"); // reset
    delay(500);
    if(Serial.find("invalid")) { // <<<<<<<<<<<------------ Depends on ESP8266 firmware. Use serial monitor to get YOUR "Ready" string
        //WiFi is ready
        delay(100);
        for(int i=0;i<5;i++){
            if(connectWiFi()){
                delay(5000);
                Serial.println("AT+CIPMUX=0"); // set to single connection mode
                delay(100);
                String cmd = "AT+CIPSTART=\"TCP\",\"";
                cmd += DST_IP;
                cmd += "\",80";
                Serial.println(cmd);
                if(Serial.find("Error")){
                    err = 3;
                    return false;
                }    
                cmd = "GET /data/2.5/forecast?id=";
                cmd += LOCATIONID;
                cmd += "&units=metric&mode=xml&APPID=";
                cmd += APPID;
                cmd += " HTTP/1.0\r\nHost: api.openweathermap.org\r\n\r\n";
                Serial.print("AT+CIPSEND=");
                Serial.println(cmd.length());
                if(!Serial.find(">")){ //connection timeout
                    Serial.println("AT+CIPCLOSE");
                    err = 4;
                    return false;
                }
                Serial.print(cmd);  //ask the forecaster
                int n = 0; // char counter
                byte itemCnt = 0;
                int temp=0;
                bool timeout=true;
                inString = "";
                //The weather data for time 12-15 are in section "<forecast>" ...
                if (SerialFind("forecast>", 6000,1000)){ 
                    //..behind time entry  T15:00:00 to="
                    if (SerialFind("T15:00:00\" to=\"", 1000,500)){ 
                        //The expected temperature behind  max="
                        if (SerialFind("max=\"", 1000,500)){ 
                            t1 = millis();
                            //The value is surrounded by "
                            while((n < 7) && (abs(millis() - t1) < 500)){
                                if (Serial.available()) {
                                    char inChar = Serial.read();
                                    if (inChar=='"'){ //End of number
                                        forecast = (int)(10 * inString.toFloat());
                                        Serial.println("AT+CIPCLOSE");
                                        return true;
                                    }
                                    inString += inChar;
                                    n++;
                                }
                            }//while
                        }else err = 7;
                    }else err = 6;
                }else err = 5;
                Serial.println("AT+CIPCLOSE");
                return false;
            }else err = 2;
        }
    }else err = 1;
    //Error: no forecast
    err = 8;
    return false;
}

//-----------------------------------------------------------------------------
void setup(){
//-----------------------------------------------------------------------------
    pinMode(13, OUTPUT); //Onboard green LED
    /*
    The led controller MAX7219 is in power-saving mode on startup,
    we have to do a wakeup call
    */  
    led.shutdown(false);
    /* Set the brightness to a low value */
    led.setIntensity(4);
    for (int i=0; i<8; i++){
        digitalWrite(13, HIGH);   // turn the green LED on 
        led.setChar(i, '8', false);  
        if (i>0) led.setChar(i-1, ' ', false);
        delay(60);  
        digitalWrite(13, LOW);   // turn the green LED off 
        delay(140);  
    }
    led.setChar(7, ' ', false);  
    sensors.begin();/* Init Dallas Temperature library */
    sensors.setResolution(TEMP_12_BIT); // Set resolution to 12-Bit
    led.setChar(7, 'S', false);  
    led.setChar(6, 'E', false);  
    led.setChar(5, 'N', false);  
    led.setChar(4, 'S', false);  
    led.setChar(3, 'O', false);  
    led.setChar(2, 'R', false);  
    Serial.begin(115200);
    Serial.setTimeout(5000);
    delay(2000);
    led.clearDisplay(); 
    led.setChar(7, '1', false);  
}
//-----------------------------------------------------------------------------
void loop(){
//-----------------------------------------------------------------------------
static int temperature;
static bool haveForecast = false;
static int tic=1;

    led.shutdown(false);
    sensors.requestTemperatures(); // get sensor temperature
    temperature = sensors.getTempCByIndex(0) * 10;
    writeIntTo7Segment(temperature, false);
    if (haveForecast){
        writeIntTo7Segment(forecast, true);
    }else{
        if (err > 0){
            led.setChar(7, 'E', false);  
            led.setChar(6, err, false);  
            led.setChar(5, '-', false);  
            led.setChar(4, '-', false);  
        }
    }
    clearSerial(1800);
  
    tic--;
    if (tic < 0){
        haveForecast = getForecast();
        if (haveForecast){
            tic = 1800; //next request in 1h
        }else{
            tic = 14; //try again in 30s
        }  
    }
}
